import copy
import cPickle
import dircache
import RsrnMs

class DNet:
    def __init__(self, patternSize, contextSize, numPatterns, learning_rate):

        self.name = "Dual Network"
        self.run = False
        self.patternSize = patternSize
        self.contextSize = contextSize
        self.numPatterns = numPatterns

        # performance network
        self.net1 = RsrnMs.RSRNMS(patternSize,
                                  contextSize,
                                  learning_rate=learning_rate)
        self.net1.createPatternSequence(sequence='A',
                                        numPatterns=numPatterns,
                                        ambiguity=(2,7),
                                        seqtype="Random")
        self.net1.createPatternSequence(sequence='B',
                                        numPatterns=numPatterns,
                                        ambiguity=(3,8),
                                        seqtype="Random")

        ## storage network
        self.net2 = None 

    def __str__(self):
        s = 'net1:\n' + str(self.net1)
        return s

    def train_sequenceA(self, numEpochs, lr_factor=1.):
        for i in range(numEpochs):
            self.net1.epoch(sequence='A', self_enforce_interleaved=0,
                            lr_factor=lr_factor, repetitions=0, fliprate=0)
    ## endof train_sequence1
    
    def train_sequenceB(self, numEpochs, numInterleaved, lr_factor=1.):
        for i in range(numEpochs):
            self.net1.epoch(sequence='B', self_enforce_interleaved=0,
                            lr_factor=lr_factor, repetitions=0, fliprate=0)
            ## interleave pseudopatterns
            for i in range(numInterleaved):
                pp_in, pp_out = self.net2.reverberate_pseudopattern(repetitions=5)
                self.net1.train_one_pattern(pp_in, pp_out, scale=1) #, lr_factor=lr_factor)
    ## endof train_sequenceB

    def transferNet1to2(self, num=10):
        if not self.net2:
            self.net2 = copy.deepcopy(self.net1)
            self.net2.set_new_weights()

            start,stop = {'A': 0, 'B': 10},{'A': 10, 'B': 20}            
            for seq in self.net1.sequenceNames:
                self.net2.init_sequence_stats(seq)

        for i in range(num):
            pp_in, pp_out = self.net1.reverberate_pseudopattern(repetitions=5)
            self.net2.train_one_pattern(pp_in, pp_out, scale=1, count=1)
    ## endof transferNet1to2

    ## storing and restoring networks
    def saveNetwork(self, savefilename=0):
        ## open file and dump object
        savedir = "" # "C:/Python24/MyScripts/savefiles/"
        newfile = file(savedir+savefilename, 'wb')
        cPickle.Pickler(newfile, 0).dump(self)
        newfile.close()

        print 'saved:', savedir+savefilename
    ## endof saveNetwork

## class methods

def loadNetwork(loadfilename=0):
    # open file and load object
    savedir = "" # "C:/Python24/MyScripts/savefiles/"
    loadfile = file(savedir+loadfilename, 'rb')
    net = cPickle.Unpickler(loadfile).load()
    loadfile.close()

    print 'loaded:', savedir+loadfilename

    return net
## endof loadNetwork

## endof DNet



