## figure plotting
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wx import NavigationToolbar2Wx as Toolbar ## Toolbar ## 
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import FigureManager

from matplotlib.figure import Figure
from matplotlib.axes import Subplot
import matplotlib.numerix as numpy

## window gui
from wx.lib.rcsizer import RowColSizer
import wx
from wx.lib import newevent
from wxPython.wx import wxSize

## network training
import DNet
import thread
import time

## network saving and loading
import cPickle
import dircache
import os

## ids
ID_MenuNew = wx.NewId()
ID_MenuSave = wx.NewId()
ID_MenuLoad = wx.NewId()
ID_MenuExit = wx.NewId()
ID_Close = wx.NewId()

ID_SaveNetwork = wx.NewId()
ID_LoadNetwork = wx.NewId()
ID_CreateNetwork = wx.NewId()
ID_PatternSize = wx.NewId()
ID_ContextSize = wx.NewId()

ID_TrainA = wx.NewId()
ID_TrainB = wx.NewId()
ID_Transfer = wx.NewId()
ID_Stop = wx.NewId()
ID_InfoSA_1 = wx.NewId()
ID_InfoSA_5 = wx.NewId()
ID_InfoSB_1 = wx.NewId()
ID_InfoSB_5 = wx.NewId()
ID_numEpochsA = wx.NewId()
ID_numEpochsB = wx.NewId()
ID_SSEA = wx.NewId()
ID_SSEB = wx.NewId()
ID_Net2 = wx.NewId()
ID_TransferLimit = wx.NewId()
ID_RunLimit = wx.NewId()
ID_Gauge = wx.NewId()

class Log:
    def WriteText(self, text):
        if text[-1:] == '\n':
            text = text[:-1]
        wx.LogMessage(text)
    write = WriteText


class DNetPanel(wx.Panel):
    def __init__(self, parent, log):
        ## the network i will train

        self.dnet = DNet.DNet(patternSize=100,
                              contextSize=50,
                              numPatterns=10,
                              learning_rate=0.01)        
        self.parent = parent
        wx.Panel.__init__(self, parent, -1)

        ## object to organize screen items
        sizer = RowColSizer()

        self.nr = nr = 1    ## network initialization row start
        self.rr = rr = nr+3 ## run limits row start
        self.sr = sr = rr+4 ## status row start
        self.cr = cr = sr+4 ## canvas row start

        ## network creation
        sizer.Add(wx.StaticText(self, -1, 'Pattern size'), flag=wx.EXPAND, pos=(nr+0,0), border=1)
        self.tc_pat = wx.TextCtrl(self, ID_PatternSize, str(self.dnet.patternSize), size=(20,25))
        sizer.Add(self.tc_pat, flag=wx.EXPAND, pos=(nr+0,2), border=1)

        sizer.Add(wx.StaticText(self, -1, 'Context size'), flag=wx.EXPAND, pos=(nr+1,0), border=1)
        self.tc_con = wx.TextCtrl(self, ID_ContextSize, str(self.dnet.contextSize))
        sizer.Add(self.tc_con, flag=wx.EXPAND, pos=(nr+1,2), border=1)

        b_create = wx.Button(self, ID_CreateNetwork, 'Create')
        sizer.Add(b_create, flag=wx.EXPAND, pos=(nr+1,3), border=1)
        self.Bind(wx.EVT_BUTTON, self.OnCreateNetwork, b_create)

        ## training limits
        sizer.Add(wx.StaticText(self, -1, "Epoch Run Limit"), flag=wx.EXPAND, pos=(rr+0,0), border=1)
        self.sc_runlimit = wx.SpinCtrl(self, ID_RunLimit, "450")
        self.sc_runlimit.SetRange(0, 10000)
        sizer.Add(self.sc_runlimit, flag=wx.EXPAND, pos=(rr+0,2), border=1)

        sizer.Add(wx.StaticText(self, -1, 'Transfer Limit'), flag=wx.EXPAND, pos=(rr+1,0), border=1)
        self.sc_transferlimit = wx.SpinCtrl(self, ID_TransferLimit, "10000")
        self.sc_transferlimit.SetRange(0, 100000)
        sizer.Add(self.sc_transferlimit, flag=wx.EXPAND, pos=(rr+1,2), border=1)
        
        sizer.Add(wx.StaticText(self, -1, 'Transfer FFBP'), flag=wx.EXPAND, pos=(rr+2,0), border=1)
        self.tc_net2 = wx.TextCtrl(self, ID_Net2, '%d' % (self.dnet.net2 and
                                                          self.dnet.net2.numOnePatterns or
                                                          0., ),
                                   style=wx.TE_READONLY)
        sizer.Add(self.tc_net2, flag=wx.EXPAND, pos=(rr+2,2), border=1)

        ## start training A or B and stop
        b_stop = wx.Button(self, ID_Stop, 'Stop')
        sizer.Add(b_stop, flag=wx.EXPAND, pos=(rr+0,3), rowspan=1, border=1)
        wx.EVT_BUTTON(self, ID_Stop, self.OnStop)

        b_transfer = wx.Button(self, ID_Transfer, 'Transfer')
        sizer.Add(b_transfer, flag=wx.EXPAND, pos=(rr+1,3), rowspan=1, border=1)
        wx.EVT_BUTTON(self, ID_Transfer, self.OnTransferNet1to2)

        b_trainA = wx.Button(self, ID_TrainA, 'Train A')
        sizer.Add(b_trainA, flag=wx.EXPAND, pos=(rr+0,5), border=1)
        wx.EVT_BUTTON(self, ID_TrainA, self.OnTrainA)

        self.b_trainB = wx.Button(self, ID_TrainB, 'Train B')
        self.b_trainB.Disable()
        sizer.Add(self.b_trainB, flag=wx.EXPAND, pos=(rr+0,6), border=1)
        wx.EVT_BUTTON(self, ID_TrainB, self.OnTrainB)

        ## training status
        sizer.Add(wx.StaticText(self, -1, 'Sequence A'), flag=wx.EXPAND, pos=(sr+0,2), border=1)
        sizer.Add(wx.StaticText(self, -1, 'Sequence B'), flag=wx.EXPAND, pos=(sr+0,3), border=1)
        sizer.Add(wx.StaticText(self, -1, 'Trained Epochs'), flag=wx.EXPAND, pos=(sr+1,0), border=1)
        self.tc_numA = wx.TextCtrl(self, ID_numEpochsA, str(self.dnet.net1.numEpochs['A']), style=wx.TE_READONLY)
        sizer.Add(self.tc_numA, flag=wx.EXPAND, pos=(sr+1,2), border=1)
        self.tc_numB = wx.TextCtrl(self, ID_numEpochsB, str(self.dnet.net1.numEpochs['B']), style=wx.TE_READONLY)
        sizer.Add(self.tc_numB, flag=wx.EXPAND, pos=(sr+1,3), border=1)

        sizer.Add(wx.StaticText(self, -1, 'Current SSE'), flag=wx.EXPAND, pos=(sr+2,0), border=1)
        self.tc_sseA = wx.TextCtrl(self, ID_SSEA, '%.3f' % (self.dnet.net1.sse(sequence='A'), ), style=wx.TE_READONLY)
        sizer.Add(self.tc_sseA, flag=wx.EXPAND, pos=(sr+2,2), border=1)
        self.tc_sseB = wx.TextCtrl(self, ID_SSEB, '%.3f' % (self.dnet.net1.sse(sequence='B'), ), style=wx.TE_READONLY)
        sizer.Add(self.tc_sseB, flag=wx.EXPAND, pos=(sr+2,3), border=1)

        b_infoSA_1 = wx.Button(self, ID_InfoSA_1, 'SA Info .1')
        sizer.Add(b_infoSA_1, flag=wx.EXPAND, pos=(nr+0,5), border=1)
        wx.EVT_BUTTON(self, ID_InfoSA_1, self.OnShowInfoSA_1)
        
        b_infoSA_5 = wx.Button(self, ID_InfoSA_5, 'SA Info .5')
        sizer.Add(b_infoSA_5, flag=wx.EXPAND, pos=(nr+1,5), border=1)
        wx.EVT_BUTTON(self, ID_InfoSA_5, self.OnShowInfoSA_5)
        
        b_infoSB_1 = wx.Button(self, ID_InfoSB_1, 'SB Info .1')
        sizer.Add(b_infoSB_1, flag=wx.EXPAND, pos=(nr+0,6), border=1)
        wx.EVT_BUTTON(self, ID_InfoSB_1, self.OnShowInfoSB_1)
        
        b_infoSB_5 = wx.Button(self, ID_InfoSB_5, 'SB Info .5')
        sizer.Add(b_infoSB_5, flag=wx.EXPAND, pos=(nr+1,6), border=1)
        wx.EVT_BUTTON(self, ID_InfoSB_5, self.OnShowInfoSB_5)

        ## gauge (progress bar) visual remaining time...
        self.gauge = wx.Gauge(self, ID_Gauge, style=wx.GA_HORIZONTAL|wx.GA_SMOOTH)
        sizer.Add(self.gauge, flag=wx.EXPAND, pos=(sr+3,0), colspan=9, border=1)
        self.gauge.Hide()
        
        ## create canvas for showing training info
        self.fig = Figure((1,1), 75) ## (5,10), 50)
        self.canvas = FigureCanvas(self, -1, self.fig)
        self.toolbar = Toolbar(self.canvas)
        self.toolbar.Realize()

        # On Windows, default frame size behaviour is incorrect
        # you don't need this under Linux
        tw, th = self.toolbar.GetSizeTuple()
        fw, fh = self.canvas.GetSizeTuple()
        self.toolbar.SetSize(wxSize(fw, th))

        # Create a figure manager to manage things
        self.figmgr = FigureManager(self.canvas, 1, self)

        # Now put all into the sizer
        sizer.Add(self.canvas, flag=wx.EXPAND, pos=(cr,0), size=(10,9), border=1)## colspan=7) ## size=(3,8))
        sizer.Add(self.toolbar, flag=wx.BOTTOM, pos=(cr+10,0), colspan=7, border=1)

        sizer.AddGrowableCol(8)
        sizer.AddGrowableRow(cr)

        # Set the layout.
        self.SetSizer(sizer)
        self.SetAutoLayout(True)
    ## endof __init__


    def callback(self):
        """ Let the running evolution tell us when a generation is done """
        wx.PostEvent(self, CallBackEvent())

    def OnCallBack(self, event):
        self.update_time_spent()
        self.update_graph()

    ## show info on the training progress
    def ShowInfo(self, event, limit=.1, sequence='A'):
        """Function to be used with OnShowInfoS*_*(). """
        ## stop training to show info, dialogbox?
        if self.dnet.run: return
        dlg = wx.MessageDialog(self,
                               str(self.dnet.net1.getStats(limit=limit,
                                                           sequence=sequence)),
                               'Notice',
                               style=wx.OK
                               | wx.ICON_INFORMATION)
        font = wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL)
        font.SetFamily(wx.FONTFAMILY_TELETYPE)
        dlg.SetFont(font)
        dlg.ShowModal()
        dlg.Destroy()
    ## endof ShowInfo

    def OnShowInfoSA_1(self, event): self.ShowInfo(event, limit=.1, sequence='A')
    def OnShowInfoSA_5(self, event): self.ShowInfo(event, limit=.5, sequence='A')
    def OnShowInfoSB_1(self, event): self.ShowInfo(event, limit=.1, sequence='B')
    def OnShowInfoSB_5(self, event): self.ShowInfo(event, limit=.5, sequence='B')


    def OnCreateNetwork(self, event):
        patternSize = int(self.tc_pat.GetValue())
        contextSize = int(self.tc_con.GetValue())
        self.dnet = DNet.DNet(patternSize,
                              contextSize,
                              numPatterns=10,
                              learning_rate=0.01)
        self.tc_numA.SetValue('%d' % (self.dnet.net1.numEpochs['A'], ))
        self.tc_numB.SetValue('%d' % (self.dnet.net1.numEpochs['B'], ))
        self.tc_sseA.SetValue('%.3f' % (self.dnet.net1.sse(sequence='A'), ))           
        self.tc_sseB.SetValue('%.3f' % (self.dnet.net1.sse(sequence='B'), ))
        self.tc_net2.SetValue('0')

        self.init_plot_data()

        
    def OnTrainA(self, event):
        if self.dnet.run: return
        self.dnet.run = True
        thread.start_new_thread(self.Train, (event, 'A'))

    def OnTrainB(self, event):
        if self.dnet.run: return
        self.dnet.run = True
        thread.start_new_thread(self.Train, (event, 'B'))


    def OnStop(self, event):
        self.dnet.run = False

    def Train(self, event, sequence='A'):
        if sequence == 'B' and not self.dnet.net2:
            dlg = wx.MessageDialog(self,
                                   "Transfer Net 1 to Net 2\nbefore you start\ntraining sequence B", 
                                   'Warning',
                                   style = wx.OK
                                   | wx.ALIGN_CENTRE )
            dlg.ShowModal()
            dlg.Destroy()
            self.dnet.run = False
            return

        runlimit = self.sc_runlimit.GetValue() - self.dnet.net1.numEpochsTotal
        current = 0

        self.gauge.SetRange(runlimit)
        self.gauge.SetValue(current)
        self.gauge.Show()
        
        while self.dnet.run and current < runlimit:
            if sequence == 'A':
                self.dnet.train_sequenceA(10, 1.)
            else:
                self.dnet.train_sequenceB(10, 10, 1.)
            current += 10
            self.gauge.SetValue(current)
            self.update_graph()

        ## the current run limit was run dry
        if self.dnet.run:
            self.sc_runlimit.SetValue(self.sc_runlimit.GetValue()*2)
        self.dnet.run = False
        self.gauge.Hide()


    def OnTransferNet1to2(self, event):
        numOne = self.dnet.net2 and self.dnet.net2.numOnePatterns or 0
        total = self.sc_transferlimit.GetValue()
        maxv = total - numOne
        info = "Transfering information from Net 1 to Net 2"

        ## check to see if net 2 should be reset
        dlg2 = wx.MessageDialog(self,
                                "Do you wish to reset Net 2?",
                                'Warning',
                                style = wx.YES_NO
                                | wx.NO_DEFAULT)

        if self.dnet.net2:
            if dlg2.ShowModal() == wx.ID_YES:
                self.tc_net2.SetValue("0")
                self.dnet.net2 = None ## reset the net
                maxv = total

        dlg = wx.ProgressDialog("Sleep State",
                                info,
                                maximum = maxv,
                                parent=self,
                                style = wx.PD_CAN_ABORT
                                | wx.PD_APP_MODAL
                                | wx.PD_ELAPSED_TIME
                                #| wx.PD_ESTIMATED_TIME
                                | wx.PD_REMAINING_TIME
                                | wx.GA_SMOOTH
                                | wx.PD_AUTO_HIDE 
                                )

        ## transfer information to net 2
        keepGoing = True
        count = 0

        while keepGoing and count < maxv:
            count += 10
            self.dnet.transferNet1to2(10)
            self.tc_net2.SetValue('%d' % (self.dnet.net2.numOnePatterns, ))
            keepGoing = dlg.Update(count, info+'\nPatterns presented: %d / %d' % (self.dnet.net2.numOnePatterns,
                                                                                  total))

        dlg.Destroy()
        if not keepGoing: 
            if dlg2.ShowModal() == wx.ID_YES:
                self.tc_net2.SetValue("0")
                self.dnet.net2 = None ## reset the net
                self.b_trainB.Disable()
        if self.dnet.net2: 
            self.b_trainB.Enable()
    ## endof OnTransferNet1to2


    def GetToolBar(self):
        # You will need to override GetToolBar if you are using an 
        # unmanaged toolbar in your frame
        return self.toolbar

    ## drawing the graphs
    def init_plot_data(self):
        """ show statistics of the evolution """
        self.cls_max  = 100 ## self.evolution.get_problem_size()

        self.fig.clear()

        ## create subplots for correct units (average)
        self.corA_plot = corA = self.fig.add_subplot(111)
        self.corB_plot = corB = self.fig.add_subplot(111)

        self.corA, = corA.plot([0], [0], 'b', lw=1.) # 2.0)
        self.corB, = corB.plot([0], [0], 'r', lw=1.) # 2.0)
        corB.set_xlim([0,205])
        corB.set_ylim([0,int(self.dnet.patternSize*1.1)])
        corB.grid('on')
        corB.set_title('Averge number of correct units (y/%d)' % (self.dnet.patternSize, ), fontsize=12)
        
        ## manage the space surrounding the subplots
        self.fig.subplots_adjust(left=.1, right=.95,
                                 bottom=.1, top=.9,
                                 wspace=.2, hspace=.4)

        # Draw the graph.
        self.canvas.draw()
        self.update_graph()


    def update_graph(self):
        # update viewport
        extra = int((self.dnet.net1.numEpochsTotal-151) / 50)
        if extra < 0: extra = 0

        self.corA_plot.set_xlim([0, 205+extra*50])

        ## CORRECT NUMBER GRAPH
        
        ## get cor graph data
        cor_ydataA1 = self.dnet.net1.stats['numcor']['A1']
        cor_xdata = range(len(cor_ydataA1))

        ## data for sequence B                           
        cor_xdataB, cor_ydataB1, cor_ydataB5 = [], [], []
        try:
            cor_ydataB1 = self.dnet.net1.stats['numcor']['B1']
        except ValueError:
            pass

        x_step = int((205+extra*50)/100)
        cor_xdata = [cor_xdata[i] for i in range(0, len(cor_xdata), x_step)]
        cor_ydataA1 = [cor_ydataA1[i] for i in range(0, len(cor_ydataA1), x_step)]
        cor_ydataB1 = [cor_ydataB1[i] for i in range(0, len(cor_ydataB1), x_step)]

        ## update graph data        
        self.corA.set_data(cor_xdata, cor_ydataA1)
        self.corB.set_data(cor_xdata, cor_ydataB1)

        # Redraw the graph.
        self.canvas.draw()
        self.canvas.gui_repaint()

        ## update text controls
        self.tc_numA.SetValue('%d' % (self.dnet.net1.numEpochs['A'], ))
        self.tc_numB.SetValue('%d' % (self.dnet.net1.numEpochs['B'], ))
        self.tc_sseA.SetValue('%.3f' % (self.dnet.net1.sse(sequence='A'), ))
        self.tc_sseB.SetValue('%.3f' % (self.dnet.net1.sse(sequence='B'), ))
        if self.dnet.net2:
            self.tc_net2.SetValue('%d' % (self.dnet.net2.numOnePatterns, ))
            self.b_trainB.Enable()
        else:
            self.tc_net2.SetValue("0")
            
            

    def OnMenuSave(self, event):
        ## file filters
        wildcard = "Saved Dual Network (*.dnet)|*.dnet|"    \
                   "Python source (*.py)|*.py|"     \
                   "All files (*.*)|*.*"

        ## open file dialog
        dlg = wx.FileDialog(self,
                            message="Save file as ...",
                            defaultDir='C:\Python24\MyScripts\savefiles\\',
                            defaultFile="",
                            wildcard=wildcard,
                            style=wx.SAVE)

        ## set default filter
        dlg.SetFilterIndex(0)

        # show the dialog and retrieve the user response. if it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            ## save this network
            self.dnet.saveNetwork(savefilename=dlg.GetPath())

        # destroy the dialog. 
        dlg.Destroy()

    def OnMenuLoad(self, event):
        ## file filters
        wildcard = "Saved Dual Network (*.dnet)|*.dnet|"    \
                   "Python source (*.py)|*.py|"     \
                   "All files (*.*)|*.*"

        ## open file dialog
        dlg = wx.FileDialog(self,
                            message="Choose a file",
                            defaultDir='C:\Python24\MyScripts\savefiles\\',
                            defaultFile="",
                            wildcard=wildcard,
                            style=wx.OPEN | wx.CHANGE_DIR)

        # show the dialog and retrieve the user response. if it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            ## load the old network
            self.dnet = DNet.loadNetwork(loadfilename=dlg.GetPath())

        ## destroy the dialog
        dlg.Destroy()
        self.init_plot_data()
## endof DNetPanel


        

class DNetFrame(wx.Frame):
    def __init__(self, parent, ID, title):
        wx.Frame.__init__(self, parent, ID, title, pos=(50,50), size=(0,0),
                          style=wx.NO_FULL_REPAINT_ON_RESIZE|wx.DEFAULT_FRAME_STYLE)


        self.CreateStatusBar()

        ## file menu
        filemenu = wx.Menu()
        filemenu.Append(ID_MenuNew, "&New\tCtrl-N", "Create a new network")
        filemenu.AppendSeparator()
        filemenu.Append(ID_MenuSave, "&Save\tCtrl-S", "Save the existing network to file")
        filemenu.Append(ID_MenuLoad, "&Load\tCtrl-L", "Load a network from file")
        filemenu.AppendSeparator()
        filemenu.Append(ID_MenuExit, "E&xit\tAlt-F4", "Exit the program without saving")

        ## add event handlers
        wx.EVT_MENU(self, ID_MenuNew, self.OnMenuNew)
        wx.EVT_MENU(self, ID_MenuSave, self.OnMenuSave)
        wx.EVT_MENU(self, ID_MenuLoad, self.OnMenuLoad)
        wx.EVT_MENU(self, ID_MenuExit, self.OnMenuExit)

        ## train menu
        trainmenu = wx.Menu()
        trainmenu.Append(ID_TrainA, "Train Net &A\tAlt-A", "Train Net A")
        trainmenu.Append(ID_TrainB, "Train Net &B\tAlt-B", "Train Net B")
        trainmenu.AppendSeparator()
        trainmenu.Append(ID_Transfer, "&Transfer Info\tAlt-T", "Transfer Info")
        trainmenu.AppendSeparator()
        trainmenu.Append(ID_Stop, "&Stop training\tAlt-S", "Stop training")

        ## event handlers should exist?
        wx.EVT_MENU(self, ID_TrainA, self.OnTrainA)
        wx.EVT_MENU(self, ID_TrainB, self.OnTrainB)
        wx.EVT_MENU(self, ID_Transfer, self.OnTransferNet1to2)
        wx.EVT_MENU(self, ID_Stop, self.OnStop)
        
        ## connect menu items
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu, "&File")
        menuBar.Append(trainmenu, "&Training")
        self.SetMenuBar(menuBar)
        self.Show(True)
        wx.EVT_CLOSE(self, self.OnCloseFrame)

        ## create the panel
        win = DNetPanel(self, Log())

        sizer = win.GetSizer()
        b_close = wx.Button(win, ID_Close, "Close")
        sizer.Add(b_close, flag=wx.EXPAND, pos=(win.sr+2,5), colspan=2, border=1)
        self.Bind(wx.EVT_BUTTON, self.OnCloseMe, b_close)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        ## handle closing
        wx.EVT_CLOSE(win, self.OnCloseFrame)
        self.SetSize((600, 700))
        win.SetFocus()
        self.window = win
        self.window.init_plot_data()


    ## functions for closing the frame
    def OnCloseMe(self, event):
        self.Close(True)

    def OnCloseWindow(self, event):
        self.Destroy()
        
                       
    def OnMenuExit(self, event):
        self.Close(True)

    def OnCloseFrame(self, event):
        if hasattr(self, "window") and hasattr(self.window, "ShutdownDemo"):
            self.window.ShutdownDemo()
        event.Skip()

    def OnCloseWindow(self, event):
        self.Destroy()

    ## functions for new, save and open
    def OnMenuNew(self, event):
        self.window.OnCreateNetwork(event)
        
    def OnMenuSave(self, event):
        self.window.OnMenuSave(event)

    def OnMenuLoad(self, event):
        self.window.OnMenuLoad(event)

    ## functions for transferring training calls
    def OnTrainA(self, event):
        self.window.OnTrainA(event)

    def OnTrainB(self, event):
        self.window.OnTrainB(event)

    def OnTransferNet1to2(self, event):
        self.window.OnTransferNet1to2(event)

    def OnStop(self, event):
        self.window.OnStop(event)


class MyApp(wx.App):
    def __init__(self, name):
        self.name = name
        wx.App.__init__(self, 0)
        

    def OnInit(self):
        wx.InitAllImageHandlers()
        wx.Log_SetActiveTarget(wx.LogStderr())

        frame = DNetFrame(None, -1, self.name)

        self.SetTopWindow(frame)
        self.frame = frame
        return True

def main():
    app = MyApp("Dual Network")
##    jarra = app.frame.window.fig, app.frame.window.canvas, app.frame.window.figmgr
    app.MainLoop()
##    return jarra

if __name__ == "__main__":
    main()
