import Numeric
import random
import copy

def binarray(size):
    """create an array with random binary values. """
    return [random.randint(0, 1) for i in range(size)]

def binarray_limit(size, limit):
    """create a non-uniform array with random binary values, but use a treshold. """
    return [random.random() > limit and 1 or 0 for i in range(size)]

def create_scalefunc(xmin, xmax):
    """create a function for scaling input vectors, the function returned is
    called with the value to be scaled: newfunc(v) -> float """
    def f(x):
        return (.9-.1)/(xmax-xmin)*x + (.9-(.9-.1)/(xmax-xmin)*xmax)
    return f

##def create_scalefunc_bin(xmin, xmax):
##    """create a function for scaling input vectors, the function returned is
##    called with the value to be scaled: newfunc(v) -> float """
##    def f(x):
##        return (1-(-1))/(xmax-xmin)*x + (1-(1-(-1))/(xmax-xmin)*xmax)
##    return f
##
##def getVertical(a, n):
##    return Numeric.transpose(a)[n]
##
##def getHorizontal(a, n):
##    return a[n]
##
def log(x):
    """ logistic transfer function. """
    return 1. / (1. + Numeric.exp(-x))

def treshold(x, l):
    """ output 0 or 1 if activation exceeds limit. """
    return x >= (1-l) and 1 or x > l and x or 0

##def tresholdChar(x, l):
##    """ output 0 or 1 if activation exceeds limit. """
##    return x >= (1-l) and '1' or x > l and 'x' or '0'
##
def ndim(n, *args):
    """
    Makes a multi-dimensional array of random floats. (Replaces RandomArray).
    """
    if not args: 
        return [random.random() for i in xrange(n)]
    A = [] 
    for i in range(n):
        A.append( ndim(*args) ) ## recursive call
    return A 

def randomArray(size, bound):
    """
    Returns an array initialized to random values between -max and max.
    """
    if type(size) == type(1):
        size = (size,)
    temp = Numeric.array(ndim(*size)) * (2.0 * bound)
    return temp - bound

##def randomBinArrayyy(size):
##    """
##    Returns an array initialized to random binary values.
##    """
##    return map(lambda x: x >= 0 and 1 or 0,
##               randomArray(size, 1))
##
##def randomBinArray(size):
##    """
##    Returns an array initialized to random binary values.
##    """
##    A = []
##    for i in range(size):
##        A.append(random.randrange(0, 2))
##    return A
##
##


def DistributedSequence(num=10, size=30, limit=.3, all=[], verbose=0):
    """create a list of arrays with a certain density.
    
      num - # binarrays

     size - length of the binarrays
    
    limit - number or percentage """
    
    ## limit as integer
    limit = type(limit) == int and limit or int(size*limit)
    
    ## use all from input
    all = copy.deepcopy(all)
    lastlen, nexttries = len(all), 0
    
    while len(all) < num:
        if lastlen != len(all):
            lastlen = len(all)
            nexttries = 0
            
        next = binarray(size)
        nexttries += 1
        if nexttries > 200:
            limit += 1
            nexttries = 0
            if verbose: print 'up the limit %s' % limit
            
        i = 0
        for othr in all:
            i += 1
            simequal = reduce(Numeric.add, Numeric.equal(next, othr))
            if simequal > limit:
                break
        else:
            if verbose: print '\nadd %s' % (i+1, )
            all.append(next)

    return all



def RandomSequence(num=10, size=100, eq_limit=(.45, .55), old=[], verbose=0):
    all = copy.deepcopy(old)
    oldnum = len(old)
    tries = 0
    eq_limit = map(lambda x: int(size*x+.5), eq_limit)

    while len(all) < num+oldnum:
        if tries > 1000:
            tries = 0
            if verbose: print 'up the limit', eq_limit,
            eq_limit = map(lambda x,y: x+y, eq_limit, (-1,1))
            if verbose: print eq_limit

        # get a new pattern
        model = binarray(size)

        for a in all:
            if not (eq_limit[0] <= sum(Numeric.equal(model, a)) <= eq_limit[1]):
                tries += 1
                break
        else:
            all.append(copy.deepcopy(model))
            if verbose: print 'add:', len(all), 'tries:', tries
            tries = 0

    return all[oldnum:]
            
        
    
def RatioSequence(num=10, size=100, eq_limit=(.6,.7), on_ratio=.65, old=[], verbose=0):
    ## move from percentage to integers
    on_ratio = int(size*on_ratio+.5)
    eq_limit = map(lambda x: int(size*x+.5), eq_limit)

    if verbose: print on_ratio, eq_limit

    model = [1]*on_ratio+[0]*(size-on_ratio)

    tries = 0

    oldnum = len(old)
    all = copy.deepcopy(old)
    
    
    while len(all) < num+oldnum:
        ## lower the acceptance limit 
        if tries > 1000:
            tries = 0
            eq_limit = map(lambda x,y: x+y, eq_limit, (-1,1))
            if verbose: print 'up the limit', eq_limit

        ## shuffle the model
        for i in range(10): random.shuffle(model)

        ## check equality
        for a in all:
            if not (eq_limit[0] <= sum(Numeric.equal(model, a)) <= eq_limit[1]):
                tries += 1
                break
        else:
            all.append(copy.deepcopy(model))
            if verbose: print 'add:', len(all), 'tries:', tries
            tries = 0

    return all[oldnum:]
