import dircache
import Numeric, random, string, copy
import cPickle
import Functions

from Numeric import array

#global scalefunc
scalefunc = Functions.create_scalefunc(xmin=.0, xmax=1.)

class RSRNMS:
    def __init__(self, *args1, **args2):
        self.initializeNetwork(*args1, **args2)
        
    def initializeNetwork(self, patternSize=0, contextSize=0,
                          learning_rate=.04, momentum=.5, useCrossEntropy=1,
                          bias=1, usescaled=1, randomize_patterns=1): 
        
        # network configuration
        self.patternSize = patternSize
        self.contextSize = contextSize
        self.autoassociativeSize = contextSize+patternSize
        self.targetSize = patternSize

        self.inputLayerSize = self.patternSize+self.contextSize
        self.hiddenLayerSize = self.contextSize
        self.outputLayerSize = self.patternSize+self.contextSize+self.targetSize

        self.numSequences = 0

        self.name = 'Reverberated Simple Recurrent Network'
        self.configuration = {
            '0. configuration': [self.inputLayerSize, self.hiddenLayerSize, self.outputLayerSize],
            '1. standard input': patternSize,
            '2. context units': contextSize,
            '3. target units': patternSize,
            '4. autoassociative output': patternSize+contextSize,
            '5. output layer size': patternSize+contextSize+patternSize}

        self.bias = bias
        self.biasUnit = bias and [-1.] or[]
        self.learning_rate = learning_rate
        self.momentum = momentum

        self.numEpochsTotal = 0
        self.numEpochs = {}

        self.numOnePatterns = 0
        self.randomize_patterns = randomize_patterns
        
        self.collect_pseudoPatterns = False
        self.pseudoPatternWaste = []
        self.use_collected_pseudoPatterns = False
        self.pseudoPatternsList = []

        # weights
        self.hiddenWeights = Functions.randomArray([self.inputLayerSize+self.bias, self.hiddenLayerSize], .5)
        self.outputWeights = Functions.randomArray([self.hiddenLayerSize+self.bias, self.autoassociativeSize+self.targetSize], .5)

        # training patterns
        self.usescaled = usescaled

        self.sequenceNames = []
        self.sequenceTypes = {}
        self.sequenceTrain = {}
        self.patternNames = {} # place list of patternnames under sequence name keywords

        self.allPatterns = []
        self.patternInput = {}
        self.patternInputScaled = {}
        self.patternContext = {}
        self.patternOutput = {}
        self.patternAmbiguity = {}

        # error calculation
        self.hiddenAct = 0
        self.hiddenDelta = 0
        self.lastdwh = 0

        self.outputAct = 0
        self.outputDelta = 0
        self.lastdwo = 0

        self.useCrossEntropy = useCrossEntropy

        # debugging
        self.verbose = 0

        # thread stopping
        self.run = False

        # gather statistics
        self.stats = {}
        self.stats['interval'] = 50
        self.stats['sse'] = {}
        self.stats['numcor'] = {}
        self.stats['numcor_full'] = {}
    # endof __init__

    def createSequences(self):
        self.createPatternSequence('A', 10, (3,6), "Random")
        self.createPatternSequence('B', 10, (2,8), "Random")
    ## endof createSequences
        

    def set_new_weights(self):
        # weights
        self.hiddenWeights = Functions.randomArray([self.inputLayerSize+self.bias, self.hiddenLayerSize], .5)
        self.outputWeights = Functions.randomArray([self.hiddenLayerSize+self.bias, self.autoassociativeSize+self.targetSize], .5)
    ## endof set_new_weights
        

    def saveNetwork(self, savefilename=0, complete=0):
        """save function that won't be used. """
        # get directory listing
        savedir = "C:/Python24/MyScripts/savefiles"
        newfilename = ''

        if complete:
            newfilename = savefilename
        elif savefilename:
            # easy
            newfilename = '%s/%s' % (savedir, savefilename)
        else:
            # new file name must be decided
            oldfiles = dircache.listdir(savedir)
            oldnetws = filter(lambda s: s.find('network') >= 0,
                              oldfiles)
            oldnums = map(int,
                          map(lambda s: s.replace('network', '').replace('.txt', ''),
                              oldnetws)) or [0]
            # set new filename
            newfilename = '%s/network%d.txt' % (savedir, max(oldnums)+1)

        print 'save', newfilename

        # open file and dump object
        newfile = file(newfilename, 'w')
        cPickle.Pickler(newfile, 0).dump(self)
        
        newfile.close()
        return newfilename
    # endof saveNetwork

    # loadNetwork is a static method found at the bottom (class method)        

    # first sequence patterns
    def addPattern(self, sequence, name, inputP, targetP):
        """add pattern for first sequence. """
        if name not in self.patternNames[sequence]:
            self.patternNames[sequence].append(name)

        self.patternInput[name] = inputP
        self.patternInputScaled[name] = map(scalefunc, inputP)
        self.patternContext[name] = [.5]*self.contextSize
        # self.patternContext[name] = list(randomBinArray(self.contextSize)) # [0]*self.contextSize # FIXME
        self.patternOutput[name] = targetP

    def addPatterns(self, sequence, *patterns):
        for p in patterns:
            self.addPattern(sequence, *p)


    def init_sequence_stats(self, sequence): 
        # set first sse
        self.stats['sse'][sequence] = [self.sse(sequence=sequence)]

        # set first cor (number of correct units)
        numcor1 = self.getNumCorrect(sequence=sequence,limit=.1)
        numcor1_avg = Numeric.average(numcor1)
        self.stats['numcor'][sequence+'1'] = [numcor1_avg]
        self.stats['numcor_full'][sequence+'1'] = [('-','----',0,0,int(numcor1_avg+.5),numcor1)]
        
        numEpochsAll = map(lambda x: self.numEpochs[x], self.sequenceNames)
        self.stats['active sequence'] = [-1]
        self.stats[(sequence, 1, self.numEpochsTotal, self.numEpochs[sequence], int(Numeric.average(numEpochsAll)))] = numcor1
    # endof init_sequence_stats


    def init_sequence_holders(self, sequence, start, stop, num): 
        if self.sequenceNames:
            start = max(self.patternNames[self.sequenceNames[-1]])+1
            stop = start + num
            self.sequenceNames.append(sequence)
            self.patternNames[sequence] = []
        else:
            self.sequenceNames = [sequence]
            self.patternNames[sequence] = []

            # patterns for all sequences are kept in the same dictionaries
            self.patternInput = {}
            self.patternInputScaled = {}
            self.patternOutput = {}
            self.patternAmbiguity = {}
        self.numEpochs[sequence] = 0

        return start, stop
    # endof init_sequence_holders

    def setSequenceInfo(self, sequence, seqtype):
        self.sequenceTypes[sequence] = seqtype
    # endof setSequenceInfo


    # create a sequence of patterns
    def createPatternSequenceRandom(self, sequence=0, numPatterns=10, ambiguity=()):
        self.createPatternSequence(sequence=sequence, numPatterns=numPatterns, ambiguity=ambiguity,
                                   seqtype="Random")

    def createPatternSequenceDistributed(self, sequence=0, numPatterns=10, ambiguity=()):
        self.createPatternSequence(sequence=sequence, numPatterns=numPatterns, ambiguity=ambiguity,
                                   seqtype="Distributed")

    def createPatternSequenceRatio(self, sequence=0, numPatterns=10, ambiguity=()):
        self.createPatternSequence(sequence=sequence, numPatterns=numPatterns, ambiguity=ambiguity,
                                   seqtype="Ratio")

    def createPatternSequence(self, sequence=0, numPatterns=10, ambiguity=(), seqtype='Random'):
        """Create a sequence of numPatterns+1 patterns to be stored in the network. """
        if not sequence: return
        if sequence in self.sequenceNames: return

        start, stop = 0, numPatterns
        start, stop = self.init_sequence_holders(sequence, start, stop, numPatterns)

        # start creating the sequence
        self.setSequenceInfo(sequence, seqtype+" Sequence")
        allp = []
        
        # get the new patterns needed
        newneeded = numPatterns+1-int(bool(ambiguity))
        newPatterns = []

        if seqtype == "Random":
            newPatterns = Functions.RandomSequence(num=numPatterns+1-int(bool(ambiguity)),
                                                   size=self.patternSize,
                                                   eq_limit=(.48,.60),
                                                   old=self.allPatterns,
                                                   verbose=1)
        elif seqtype == "Distributed":
            newPatterns = Functions.DistributedSequence(num=len(self.allPatterns)+newneeded,
                                                        size=self.patternSize,
                                                        limit=.45,
                                                        all=self.allPatterns,
                                                        verbose=1)[-newneeded:] # this function returns list of old+new
        elif seqtype == "Ratio":
            newPatterns = Functions.RatioSequence(num=numPatterns+1-int(bool(ambiguity)),
                                                  size=self.patternSize,
                                                  eq_limit=(.5,.6),
                                                  on_ratio=.5,
                                                  old=self.allPatterns,
                                                  verbose=1)
        elif seqtype == "Truly Random":
            newnum = numPatterns+1-int(bool(ambiguity))
            newPatterns = [Functions.binarray(self.patternSize) for i in range(newnum)]
        else:
            print 'no sequence type match'
            return
            
        self.allPatterns += copy.deepcopy(newPatterns)
        
        # check if ambiguity is needed -> duplicate pattern
        pos = ()
        if ambiguity:
            pos = type(ambiguity) == tuple and ambiguity or (3,7)
            n_one, n_two = pos[0], pos[1]
            newPatterns = (newPatterns[:n_two] +
                           [copy.deepcopy(newPatterns[n_one])] +
                           newPatterns[n_two:])
        self.patternAmbiguity[sequence] = (pos, map(lambda x: x+start, pos))

        # start creating the sequence
        ip, op = 0, newPatterns.pop(0)
        for name in range(start, stop):
            ip, op = op, newPatterns.pop(0)
            self.addPattern(sequence, name, ip, copy.deepcopy(op))

        # keep track of the number of sequences
        self.numSequences += 1
        
        self.init_sequence_stats(sequence)
    # endof createPatternSequence
            

    def collectPseudoPatterns(self, numPatterns=100, repetitions=5, fliprate=0):
        self.pseudoPatternWaste = []
        self.use_collected_pseudoPatterns = True
        self.pseudoPatternsList = []
        for i in range(numPatterns):
            self.pseudoPatternsList.append(self.reverberate_pseudopattern(repetitions=repetitions,
                                                                          fliprate=fliprate))
        for seq in self.sequenceNames:
            if not self.numEpochs[seq]:
                self.sequenceTrain[seq] = ('PseudoStore %d' % (len(self.pseudoPatternsList)),
                                           '?')
    # endof collectPseudoPatterns

    def updatePseudoPatterns(self, repetitions=5, fliprate=0):
        # throw away one of the old patterns
        self.pseudoPatternWaste.append(self.pseudoPatternsList.pop(random.randrange(0, len(self.pseudoPatternsList))))
        # add a new one
        self.pseudoPatternsList.append(self.reverberate_pseudopattern(repetitions=repetitions,
                                                                      fliprate=fliprate))
    # endof updatePseudoPatterns

    
    def get_pseudoPattern(self):
        index = random.randrange(0, len(self.pseudoPatternsList))
        return self.pseudoPatternsList[index]
    # endof get_pseudoPattern


    def addAllPatterns(self):
        self.allPatterns = []
        self.patternAmbiguity = {}

        newlen, oldlen = 0, 0

        for seq in self.sequenceNames:
            oldlen = newlen
            for name in self.patternNames[seq]:
                p = self.patternInput[name]
                if p not in self.patternAllList:
                    self.allPatterns.append(p)
            m = max(self.patternNames[seq])
            self.allPatterns.append(self.patternOutput[m])

            newlen = len(self.patternAllList)
            if (newlen - oldlen) != (len(self.patternNames[seq])+1):
                self.patternAmbiguity[seq] = ((3,7), (3+m,7+m))
            else:
                self.patternAmbiguity[seq] = ()
    # endof addAllPatterns
            

    def __str__(self):
        s = self.name
        s += '\n\nconfiguration:\n'
        keys = self.configuration.keys()
        keys.sort()
        for key in keys:
            s += ' - %s: %s\n' % (key[3:].ljust(25), str(self.configuration[key]))
        s += ' - learning_rate            : %.2f\n' % (self.learning_rate, )
        
        s += '\nnum epochs:    ' + str(self.numEpochsTotal)
        s1, s2, s3, s4 = '', '', '', ''
        for name in self.sequenceNames:
            s1 += '\nnum epochs %s:  %s' % (name, str(self.numEpochs[name]))
            s2 += '\ncurrent sse %s: %s' % (name, str(self.sse(sequence=name)))
            
            numcor1 = self.getNumCorrect(sequence=name, limit=.1)
            hlp = map(lambda x: str(x).rjust(3, ' '), numcor1)
            hlp = '[' + ' '.join(hlp) + ']'
            s3 += '\ncorrect .1 %s:  %s %s' % (name, hlp, ('<'+str(int(Numeric.average(numcor1)))+'>').rjust(5, ' '))

            numcor5 = self.getNumCorrect(sequence=name, limit=.5)
            hlp = map(lambda x: str(x).rjust(3, ' '), numcor5)
            hlp = '[' + ' '.join(hlp) + ']'
            s4 += '\ncorrect .5 %s:  %s %s' % (name, hlp, ('<'+str(int(Numeric.average(numcor5)))+'>').rjust(5, ' '))

        s += s1+s2+s3+s4
        return s
    # endof __str__

    def propagate_full_sequence(self, seq='A'):
        ps = copy.deepcopy(self.patternNames[seq])
        ps.sort(reverse=True)

        # first must be done "by hand"
        i = ps.pop()
        numcor = self.propagate_input_simple(self.patternInput[i]+self.patternContext[i],i)
        numcor_list = [numcor]

        while ps:
            print numcor_list
            print i,
            i = ps.pop()
            numcor = self.propagate_input_simple([],i) # self.patternInput[i],i)
            numcor_list.append(numcor)
        print i

        # return the list with numcors
        return numcor_list

    def getLastNumcor(self, seq='A'):
        return self.stats['numcor_full'][seq+'1'][-1][-1]
        

    def propagate_input_simple(self, inp, p):
        """calculate output for any pattern, return a list. """

        # copy input or get from last activation
        if inp: inp = copy.deepcopy(inp)
        if not inp: inp = map(lambda x: x >= .5 and 1 or 0, self.outputAct[-self.patternSize:])

        if len(inp) != self.autoassociativeSize:
            inp += self.hiddenAct

        # calculate hidden activation
        self.hiddenAct = Numeric.dot(self.biasUnit+inp, self.hiddenWeights)
        self.hiddenAct = list(Functions.log(self.hiddenAct))

        # output activation, before tresholding
        self.outputAct = Numeric.dot(self.biasUnit+self.hiddenAct, self.outputWeights)
        self.outputAct = list(Functions.log(self.outputAct))

        targetOutput = list(map(lambda x: Functions.treshold(x, .1), self.outputAct[-self.targetSize:]))
        target = list(self.patternOutput[p])

        numcor = Numeric.add.reduce(map(lambda x,y: x==y, targetOutput,target))

        return numcor # target, self.outputAct
        
    def propagate_input(self, inp):
        """calculate output for any pattern, return a list. """

        # print Numeric.shape(self.biasUnit+inp), Numeric.shape(self.hiddenWeights)
        self.hiddenAct = Numeric.dot(self.biasUnit+inp, self.hiddenWeights)
        self.hiddenAct = list(Functions.log(self.hiddenAct))

        self.outputAct = Numeric.dot(self.biasUnit+self.hiddenAct, self.outputWeights)
        self.outputAct = list(Functions.log(self.outputAct))

        return self.outputAct
    # endof propagate_input


    def propagate_list(self, p):
        """return output as list without tresholding. """
        # set the input pattern
        allp = reduce(lambda x,y: x+y, self.patternNames.values())

        if p in allp:
            inp = self.usescaled and self.patternInputScaled[p] or self.patternInput[p]
            try: con = self.patternContext[p]
            except: con = self.hiddenAct
        else: return []

        res = self.propagate_input(inp+con)
        # hidden activation -> context for the next pattern
        if verbose > 1: print 'hidden activation:', array(self.hiddenAct)
        self.patternContext[p+1] = copy.deepcopy(self.hiddenAct)
        if self.verbose > 0:
            print 'input', inp+con
        return res
    # endof propagate_list


    def propagate_array(self, p):
        """return output as array. """
        
        return array(self.propagate_list(p))
    # endof propagate_array


    def propagate_treshold(self, p, limit=.1):
        """return tresholded output as list of 0 and 1. """

        res = self.propagate_array(p)

        return (map(lambda x: Functions.treshold(x, limit), res[:self.patternSize]) +
                list(res[self.patternSize:-self.patternSize]) +
                map(lambda x: Functions.treshold(x, limit), res[-self.patternSize:]))
    
    # endof propagate_treshold


    def propagate_tresholdChar(self, p, limit=.1):
        """return tresholded output as list of char. """

        return map(lambda x: tresholdChar(x, limit), self.propagate_array(p))
    # endof propagate_tresholdChar


    def propagate_tresholdStr(self, p, limit=.1, r=0):
        """return tresholded output as string. """


        if r: return ''.join(map(lambda x: tresholdChar(x, limit), self.propagate_array(p)[-r:]))
        else: return ''.join(map(lambda x: tresholdChar(x, limit), self.propagate_array(p)[r[0]:r[1]]))
    # endof propagate_tresholdStr


    def propagate_tresholdStrAll(self, limit=.1, sequence=0):
        """return tresholded output of all patterns, as a string. """
        patterns = self.patternNames[sequence]
        patterns.sort()
        s = ''
        for key in patterns:
            s += (string.rjust(str(key)+': ', 5) +
                  self.propagate_tresholdStr(key, limit) +
                  ' --> %s %d\n' % (self.propagate_checkCorrect(key, limit) and 'C' or 'x',
                                    self.propagate_checkCorrectNum(key, limit)))
        return s
    # endof propagate_tresholdStrAll


    def propagate_checkCorrectAll(self, limit=.1, sequence=0):
        """check whithin a sequence which patterns are correct. """
        patterns = self.patternNames[sequence]
        patterns.sort()
        s = ''
        for key in patterns:
            s += self.propagate_checkCorrect(key, limit) and 'C' or 'x'
        return s
    # endof propagate_checkCorrectAll


    def propagate_checkCorrect(self, p, limit=.1):
        """check if the target is correct (for one pattern). """
        result = array(self.propagate_treshold(p, limit))
        target = array(self.patternInput[p]+self.patternContext[p]+self.patternOutput[p])
        return reduce(Numeric.logical_and, result==target)
    # endof propagate_checkCorrect
        

    def propagate_checkCorrectNum(self, p, limit=.1):
        """check the number of correct target units (for one pattern). """
        result = array(self.propagate_treshold(p, limit))
        target = array(self.patternInput[p]+self.patternContext[p]+self.patternOutput[p])
        return reduce(Numeric.add, result==target)
    # endof propagate_checkCorrectNum


    def propagate_checkTargetCorrect(self, p, limit=.1):
        """check if the target pattern is correct (nevermind autoassociative units). """
        result = array(self.propagate_treshold(p, limit))[-self.targetSize:]
        target = array(self.patternOutput[p])
        return reduce(Numeric.logical_and, result==target)
    # endof propagate_checkTargetCorrect
        

    def propagate_checkTargetCorrectNum(self, p, limit=.1):
        """check the number of correct target units (nevermind autoassociative units). """
        result = array(self.propagate_treshold(p, limit))[-self.targetSize:]
        target = array(self.patternOutput[p])
        return reduce(Numeric.add, result==target)
    # endof propagate_checkTargetCorrectNum


    def getNumCorrectRange(self, sequence=0, limit=.1, start=-1, stop=-1):
        """returns list for each pattern. """
        if start < 0: start = self.patternSize+self.contextSize
        if stop < 0: stop = start+self.patternSize

        ps = self.patternNames[sequence]
        ps.sort()

        numcor = []

        for p in ps:
            result = array(self.propagate_treshold(p, limit)[start:stop])
            target = array((self.patternInput[p]+self.patternContext[p]+self.patternOutput[p])[start:stop])
            numcor.append(reduce(Numeric.add, result==target))

        return map(int, numcor)
    # endof getNumCorrectRange
        

    def getNumCorrect(self, sequence=0, limit=.1, start=-1, stop=-1):
        """auxilary function for the range one above...""" 
        return self.getNumCorrectRange(sequence, limit, start, stop)
    # endof getNumCorrect


    def getNumCorrectStr(self, sequence=0, limit=.1):
        """returns list for each pattern. """
        return ' '.join(map(str, self.getNumCorrect(sequence=sequence,
                                                    limit=limit)))
    # endof getNumCorrectStr

    def getStatsNumcorfull(self, seq='A',step=10, commalize=0, invert=0):
        s = ''
        if not invert: 
            for line in self.stats['numcor_full'][seq+'1']:
                if line[2] % step != 0: continue
                ss = '%s: <%s %4s>, %3s, %3s, %3s, [%3d], [%s]' % (seq, line[0], line[1],
                                                                   line[2], line[3], line[2]-line[3],
                                                                   line[4],
                                                                   ','.join(map(lambda x: '%3d' % x, line[5])))
                s += ss +'\n'
                print ss
        else: 
            for line in self.stats['numcor_full'][seq+'1']:
                if line[2] % step != 0: continue
                ss = '%s: <%s %4s>, %3s, %3s, %3s, [%3d], [%s]' % (seq, line[0], line[1],
                                                                   line[2], line[3], line[2]-line[3],
                                                                   line[4],
                                                                   ','.join(map(lambda x: '%3d' % (invert-x), line[5])))
                s += ss +'\n'
                print ss

        if commalize:
            s = s.replace(': ', ':, ')
            s = s.replace('[', '[,')
            s = s.replace(']', ',]')
        return s


    def getStats(self, limit=.1, sequence=0):
        seq = sequence
        limit = int(limit*10)

        def filter_func(x):
            try: return len(x) == 5 # FIXME, if this stats_key is changed
            except: return False
            
        keys = filter(filter_func, self.stats.keys())
        keys.sort()
        del filter_func        

        res = 'Information for Sequence %s:\n\n' % seq
        size1, size2 = len(str(self.numEpochsTotal)), len(str(self.patternSize))
        for key in keys:
            # content of key: 
            # sequence, limit, numEpochsTotal, numEpochs this sequence, numEpochs other sequence
            s, l, n, nS, nO = key
            if seq == s and limit == l:
                res += '(%s) -->  ' % (', '.join(map(lambda x: str(x).rjust(size1, '_'),
                                                     (n, nS, nO))), )
                                
                res += '  '.join(map(lambda x: str(x).rjust(size2, '_'),
                                     self.stats[key]))
                res += '\n'
        return res
    # endof getStats
    
    
    def epoch(self, sequence=0, self_enforce_interleaved=0, lr_factor=1., repetitions=5, fliprate=0):
        """run one epoch, randomize patterns and update the weights.

        lr_factor - learning rate factor for the training pattern"""

        # used for self_enforcing
        # lr_factor_netw: the factor used for pseudopatterns
        # lr_factor_this: the factor used for ordinary patterns
        lr_factor_this = lr_factor_netw = 1.
        lr_factor_this *= lr_factor
        if self_enforce_interleaved:
            if self.use_collected_pseudoPatterns:
                self.updatePseudoPatterns()
        
        patternNames = self.patternNames[sequence]
        
        # randomize the presentation order of the training patterns
        if self.randomize_patterns: random.shuffle(patternNames)

        # calculate activations
        for p in patternNames:
            # calculate activations
            self.propagate_list(p)
            
            # set variables for backprop_pass
            ti = array(self.patternInput[p]+self.patternContext[p]+self.patternOutput[p])
            oi = array(self.outputAct)
            inp = self.usescaled and self.patternInputScaled[p] or self.patternInput[p]
            con = self.patternContext[p]
            # update weights
            self.backprop_pass(target=ti, output=oi, inputAct=inp, context=con, lr_factor=lr_factor_this)
            if self_enforce_interleaved:
                if self.use_collected_pseudoPatterns:
                    pat = self.get_pseudoPattern()
                    self.train_one_pattern(pat[0], pat[1], scale=self.usescaled, lr_factor=lr_factor_netw)
                else:
                    self.self_enforce(repetitions=repetitions, lr_factor=lr_factor_netw, fliprate=fliprate)

        # update epochs
        self.numEpochsTotal += 1
        self.numEpochs[sequence] += 1

        # STATISTICS

        numEpochsAll = map(lambda x: self.numEpochs[x], self.sequenceNames)
        
        for seq in self.sequenceNames:
            numcor1 = self.getNumCorrect(sequence=seq, limit=.1)
            numcor1_avg = Numeric.average(numcor1)
            self.stats['numcor'][seq+'1'].append(numcor1_avg)
            if self.numEpochsTotal % 10 == 0:
                self.stats['numcor_full'][seq+'1'].append((sequence,
                                                           '%.2f' % lr_factor,
                                                           self.numEpochsTotal,
                                                           self.numEpochs[seq],
                                                           int(numcor1_avg+.5), 
                                                           numcor1))

            self.stats['sse'][seq].append(self.sse(sequence=seq))
            self.stats['active sequence'].append(seq)

            if self.numEpochsTotal % self.stats['interval'] == 0:
                self.stats[(seq, 1, self.numEpochsTotal, self.numEpochs[seq], int(Numeric.average(numEpochsAll)))] = numcor1
    # endof epoch


    def train_one_pattern(self, p_input, p_output, scale=0, lr_factor=1., count=0):
        """ update weights for one pattern.

        p_input - input pattern, input and context units

        p_output - the entire target output, autoass and output units. """

        if scale: p_input = map(scalefunc, p_input)
        
        self.propagate_input(p_input)

        ti = array(p_output)
        oi = array(self.outputAct)
        inp = list(p_input[:self.patternSize])
        con = list(p_input[self.patternSize:])

        self.backprop_pass(target=ti,
                           output=oi,
                           inputAct=inp,
                           context=con,
                           lr_factor=lr_factor)
        if count: self.numOnePatterns += 1
    # endof train_one_pattern
        

    def backprop_pass(self, target, output, inputAct, context, lr_factor=1.):
        """calculate actual output and update weights according to target for the given pattern. """
        ti, oi, inp, con = target, output, inputAct, context
        lr = lr_factor * self.learning_rate
        mm = lr_factor * self.momentum

        # calculate the delta term at the output layer        
        if self.useCrossEntropy: self.outputDelta = ti-oi # cross entropy (be careful with the learning rate)
        else: self.outputDelta = (ti-oi)*oi*(1-oi) # ordinary sigmoid error function
        
        # calculate hidden error
        sumk = map(lambda x: Numeric.add.reduce(self.outputDelta *
                                                self.outputWeights[x+self.bias]),
                   range(self.hiddenLayerSize))
        oj = array(self.hiddenAct)
        self.hiddenDelta = sumk*oj*(1-oj)

        # update output weights
        oik = array(map(lambda x: [x], self.biasUnit+self.hiddenAct))
        dwo = (lr * oik * self.outputDelta +
               mm * self.lastdwo)
        self.outputWeights = self.outputWeights + dwo

        # update hidden weights
        ojk = array(map(lambda x: [x], self.biasUnit+inp+con))
        dwh = (lr * ojk * self.hiddenDelta +
               mm * self.lastdwh)
        self.hiddenWeights = self.hiddenWeights + dwh

        # set values for use in training next pattern
        self.lastdwo, self.lastdwh = dwo, dwh
    # endof backprop_pass
        

    def sse(self, sequence=0):
        """calculate sum of squared errors. """
        patternNames = self.patternNames[sequence]

        # one list item per output unit
        sumSquaredError = array([0.] * self.outputLayerSize)
        
        for p in patternNames:
            self.propagate_list(p)

            outP = array(self.patternInput[p]+self.patternContext[p]+self.patternOutput[p])
            outA = array(self.outputAct)
            
            sumSquaredError += (outP - outA) ** 2

        return Numeric.add.reduce(sumSquaredError)
    # endof sse

    def train(self, sequence=0, numEpochs=100, self_enforce=0, lr_factor=1., repetitions=5, fliprate=0, verbose=0):
        """Parameters to function train()
        sequence=0,     # name of sequence to train on
        numEpochs=100,  # number of epochs to train
        self_enforce=0, # type of memory activity
        lr_factor=1.,   # learning rate factor, relative to learning rate
        repetitions=5,  # how many times to reverberate pseudopatterns
        fliprate=0,     # percentage of pseudopattern bits to flip
        verbose=0       # output level. 
        """

        interval = (numEpochs/5) or 1
        interval = interval >= 5 and interval or 10

        self.sequenceTrain[sequence] = ((not self_enforce and "Nothing") or
                                        (self_enforce and self.use_collected_pseudoPatterns and "PseudoStore "+str(len(self.pseudoPatternsList))) or
                                        "Self Enforce",
                                        lr_factor)

        if self_enforce > 1 and not self.pseudoPatternsList: 
            print '****************************\npseudopatterns had to be collected\n****************************'
            if self_enforce >= 4: self.collectPseudoPatterns(100, repetitions, fliprate)
            elif self_enforce >= 3: self.collectPseudoPatterns(50, repetitions, fliprate)
            else: self.collectPseudoPatterns(20, repetitions, fliprate)

        for i in range(1, numEpochs+1):
            if self_enforce:
                self.epoch(sequence=sequence,
                           self_enforce_interleaved=self_enforce,
                           lr_factor=lr_factor,
                           repetitions=repetitions,
                           fliprate=fliprate)
            else:
                self.epoch(sequence, 0, lr_factor=lr_factor) # nevermind input parameters, train with defaults
            if i % interval == 0:
                if verbose > 1: print self
                elif verbose > 0: print '%4d: %.4f' % (i, self.sse(sequence))
    # endof train

    def reimprint(self, repetitions=5, lr_factor=1., fliprate=0):
        self.self_enforce(repetitions, lr_factor, fliprate)
    # endof reimprint

        
    def self_enforce(self, repetitions=5, lr_factor=1., fliprate=0):
        """create reverberated pseudopattern and train the network on it. """
        inp, outp = self.reverberate_pseudopattern(repetitions=repetitions, fliprate=fliprate)
            
        self.train_one_pattern(inp, outp, scale=self.usescaled, lr_factor=lr_factor)
    # endof self_enforce


    # functions for use with GUI
    def set_callback(self, callback):
        self.callback = callback

    def get_statistics(self):
        stats = {}
        
        return stats
    # endof get_statistics


    def reverberate_pseudopattern(self, repetitions=5, fliprate=0):
        """create a pseudopattern, reverberate it so it becomes an attractor. """
        
        pSize = self.autoassociativeSize
        flipi = range(pSize)
        
        pseudopattern = Functions.binarray(pSize)
        l = lambda x: x > .5 and 1 or 0

        for i in range(repetitions):
            # propagate for activations
            pseudopattern = self.propagate_input(pseudopattern)[:pSize]
            pseudopattern = map(l, pseudopattern[:self.patternSize])+pseudopattern[self.patternSize:]

            # flip a certain percentage of the bits
            if fliprate:
                random.shuffle(flipi)
                for j in flipi[:int(pSize*fliprate+.5)]:
                    pseudopattern[j] = int(not pseudopattern[j])
        # return input output pair
        pseudo_out = self.propagate_input(pseudopattern)
        pseudo_out = (map(l, pseudo_out[:self.patternSize]) +
                      pseudo_out[self.patternSize:-self.targetSize] +
                      map(l, pseudo_out[-self.targetSize:]))
        return pseudopattern, pseudo_out
    # endof reverberate_pseudopattern


    def compare_pseudopattern(self, new, inputSize=0, checklist_keys=[], verbose=0):
        """ compare a pseudopattern to all of the input patterns. """
        checklist_keys = checklist_keys or self.patternInput.keys()
        complist = [0]*(max(checklist_keys)+1)

        for key in checklist_keys:
            check_autoassoc = self.patternInput[key][:inputSize]
            # set the number of equal positions for pattern
            eqnum = Numeric.add.reduce(Numeric.equal(check_autoassoc, new[:inputSize]))
            complist[key] = eqnum

        # gather all positions with max value
        maxv = max(complist)
        maxis = [complist.index(maxv)]
        try:
            while True: maxis.append(complist.index(maxv, maxis[-1]+1))
        except ValueError: pass
        
        # select a random max value position
        # find the best context match
        maxic = []
        for i in range(len(maxis)): # m in maxis:
            coneqnum = Numeric.add.reduce(map(lambda x,y: abs(x-y)**2, new[inputSize:], self.patternContext[maxis[i]]))
            maxic.append(coneqnum)
                    
        maxi = maxis[0]
        if maxic: maxi = maxis[maxic.index(min(maxic))]
        
        # return the result 
        return [maxi, complist[maxi]], complist, maxis
    # endof compare_pseudopattern


    def create_internal_pseudopattern_statistics(self, sequence): # repetitions=5, fliprate=0):
        """create statistics for the internal pseudopatterns in storage. """

        checklist_keys = []

        try: checklist_keys = self.patternNames[sequence]
        except: checklist_keys = self.patternNames.values()

        distancelist = [[]]
        for i in range(max(checklist_keys)):
            distancelist.append([])
        complist = [0]*(max(checklist_keys)+1)
        
        for colpat in self.pseudoPatternsList:
            # keep only input
            pat = colpat[0]
            # drop the autoassociative units
            pat = pat[:self.patternSize+self.contextSize]
            # find its match
            bestmatchi, bestmatchn = self.compare_pseudopattern(new=pat, inputSize=self.patternSize,
                                                                checklist_keys=checkdict_keys)[0]
            # increment this patterns hit number
            try:
                complist[bestmatchi] += 1
                distancelist[bestmatchi].append(bestmatchn)
            except:
                print 'exception'
        return complist, distancelist
    # endof create_pseudopattern_statistics


    def show_internal_pseudopattern_statistics(self, sequence=False, numCompPatterns=100,
                                               repetitions=5, fliprate=0, summarize=1, verbose=0):
        keys = {'repetitions': repetitions,
                'fliprate': fliprate}
        res = []
        if sequence:
            keys['sequence'] = sequence
            res = self.create_internal_pseudopattern_statistics(sequence=sequence)
        else:
            keys['numCompPatterns'] = numCompPatterns
            res = self.create_pseudopattern_statistics(**keys)
        
        for i in range(len(res[0])):
            distance = res[1][i]
            distance.sort(reverse=True)
            if summarize:
                distance_dict = {}
                for j in distance:
                    if distance_dict.has_key(j): distance_dict[j] += 1
                    else: distance_dict[j] = 1
                distance = distance_dict
            
            print '%2d: %3d, ' % (i, res[0][i]),
            if type(distance) == list: print distance
            else:
                keys = distance.keys()
                keys.sort(reverse=True)
                dct_str = '{'
                for key in keys: dct_str += '%3d: %3d, ' % (key, distance[key])
                if len(dct_str) > 1: dct_str = dct_str[:-2]+'}'
                else: dct_str += '}'
            print dct_str[:108] # FIXME, unnga linjeskift...
    # endof show_internal_pseudopattern_statistics



    def create_pseudopattern_statistics(self, sequence='A', numCompPatterns=1000, repetitions=5, fliprate=0):
        """first test, only need to be closest, set score demand later.
        sequence -        the sequence that should be compared, False for all sequences
        numCompPatterns - how many pseudopatterns to create
        repetions -       number of reverberations
        fliprate -        percentage of bits to flip. """

        # decide which input patterns to compare with
        checkdict_keys = []
        try: checkdict_keys = self.patternNames[sequence] # only one sequence
        except: checkdict_keys = self.patternInput.keys() # all sequences

        # keep a dict for each input pattern
        # keep track of how similar the pseudopatterns are
        # input pattern must be selected as the closest for score recording
        distancedict, compdict = {}, {}
        for i in checkdict_keys:
            distancedict[i],compdict[i] = [], 0

        for i in range(numCompPatterns):
            # create new pseudopattern, keep only input

            pat = self.reverberate_pseudopattern(repetitions=repetitions, fliprate=fliprate)[0]

            # find its best match
            bestmatchi, bestmatchn = self.compare_pseudopattern(new=pat, inputSize=self.patternSize,
                                                                checklist_keys=checkdict_keys)[0]

            # increment this pattern's hit number
            compdict[bestmatchi] += 1
            # append the distance
            distancedict[bestmatchi].append(bestmatchn)

        # return both dicts
        return compdict, distancedict
    # endof create_pseudopattern_statistics


    def show_pseudopattern_statistics(self, sequence=False, numCompPatterns=100,
                                      repetitions=5, fliprate=0, summarize=1, abridge=1, verbose=0):
        # create statistics
        res = self.create_pseudopattern_statistics(sequence=sequence,
                                                   numCompPatterns=numCompPatterns,
                                                   repetitions=repetitions,
                                                   fliprate=fliprate)
        # two dictionaries are returned
        compdict, distancedict = res
        checkdict_keys = compdict.keys()
        checkdict_keys.sort() # not necessary to sort

        for i in checkdict_keys:
            # get the list of distances for each input pattern
            # sort and reverse it, descending order
            distance = distancedict[i]
            distance.sort(reverse=True)

            # list of distance -> dictionary with sum for each distance
            if summarize:
                new_distancedict = {}
                for j in distance:
                    if new_distancedict.has_key(j): new_distancedict[j] += 1
                    else: new_distancedict[j] = 1
                distance = new_distancedict

            # print the result
            mark = sequence and i in self.patternAmbiguity[sequence][1] and '*' or ' '
            print '%s%2d: %3d, ' % (mark, i, compdict[i]), 

            # for summarize == False
            if type(distance) == list: print distance

            # for summarize == True
            else:
                keys = distance.keys()
                keys.sort(reverse=True)
                dct_str = '{'
                for key in keys: dct_str += '%3d: %3d, ' % (key, distance[key])
                if len(dct_str) > 1: dct_str = dct_str[:-2]+'}'
                else: dct_str += '}'
            # print the result to screen 
            if abridge: print dct_str[:108] 
            else: print dct_str
    # endof show_pseudopattern_statistics

# endof RSRN    


# class methods

def loadNetwork(savefilename=0, complete=0):
    savedir = "C:/Python24/MyScripts/savefiles"
    openfilename = ''

    if complete:
        openfilename = savefilename
    elif savefilename:
        openfilename = '%s/%s' % (savedir, savefilename)
    else: 
        oldfiles = dircache.listdir(savedir)
        oldnetws = filter(lambda s: s.find('network') >= 0,
                          oldfiles)
        oldnums = map(int,
                      map(lambda s: s.replace('network', '').replace('.txt', ''),
                          oldnetws))
        # set new filename
        openfilename = '%s/network%d.txt' % (savedir, max(oldnums))

    print 'load:', openfilename
    openfile = file(openfilename, 'r')
    net = cPickle.Unpickler(openfile).load()
    openfile.close()

    net.collect_pseudoPatterns = False
    net.use_collected_pseudoPatterns = False
    net.pseudoPatternsList = []

    return net
# endof loadNetwork
