## figure plotting
import matplotlib
matplotlib.use('WXAgg')
from matplotlib.backends.backend_wx import NavigationToolbar2Wx as Toolbar ## Toolbar ## 
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.backends.backend_wxagg import FigureManager

from matplotlib.figure import Figure
from matplotlib.axes import Subplot
import matplotlib.numerix as numpy

## window gui
from wx.lib.rcsizer import RowColSizer
import wx
from wx.lib import newevent
from wxPython.wx import wxSize

## network training
import RsrnMs
import thread
import time
import random

## network saving and loading
import cPickle
import dircache
import os

## ids
ID_MenuNew = wx.NewId()
ID_MenuSave = wx.NewId()
ID_MenuLoad = wx.NewId()
ID_MenuExit = wx.NewId()
ID_Close = wx.NewId()

ID_SaveNetwork = wx.NewId()
ID_LoadNetwork = wx.NewId()
ID_AutoSave = wx.NewId()
ID_TestN = wx.NewId()
ID_CreateNetwork = wx.NewId()
ID_PatternSize = wx.NewId()
ID_ContextSize = wx.NewId()
ID_NumSequences = wx.NewId()
ID_LearningRate = wx.NewId()
ID_SequenceType = wx.NewId()

ID_Train = wx.NewId()
ID_Stop = wx.NewId()

ID_SelfEnforceList = []
ID_LRFactorList = []
ID_NumEpochsList = []
ID_SSEList = []
ID_RunLimitList = []
for i in range(5):
    ID_SelfEnforceList.append(wx.NewId())
    ID_LRFactorList.append(wx.NewId())
    ID_NumEpochsList.append(wx.NewId())
    ID_SSEList.append(wx.NewId())
    ID_RunLimitList.append(wx.NewId())
    

ID_Gauge = wx.NewId()

## for creating pattern sequences
global_sequenceNames = 'ABCDEFGHIJKLMNOPQRSTUVWXY'
global_lr_factor = [1., .25, .25, .25, .25, .25] ## .5, .38, .27, .21, .16, .12, .9, .7]
global_self_enforce = [0, 1, 2, 1, 2] ## , 1, 1, 1, 1, 1, 1, 1, 1]
global_self_enforce_method = ["Nothing","Self Enforce","PseudoStore 20","Self Enforce","PseudoStore 20"]
global_se_methods = ["Nothing","Self Enforce","PseudoStore 20","PseudoStore 50","PseudoStore 100"]
global_runlimit = [250, 400, 800, 500, 500, 500]
global_plot_color = ['b', 'r', 'g', 'y', 'c', 'm', 'k']
global_foreground_color = ['Blue', 'Red', 'Green', 'Yellow', 'Cyan', 'Magenta', 'Black']

class Log:
    def WriteText(self, text):
        if text[-1:] == '\n':
            text = text[:-1]
        wx.LogMessage(text)
    write = WriteText


class RsrnMsPanel(wx.Panel):
    def __init__(self, parent, log):
        ## DECIDE ON ONE OR TWO PLOTS, I.E. the 0.1 limit plot...
        self.useoneplot = True

        ## network and pattern creation help
        self.network = 0
        self.sequenceNames = []
        self.lr_factor = []
        self.self_enforce = []

        ## wxpython
        self.parent = parent
        wx.Panel.__init__(self, parent, -1)

        ## object to organize screen items
        sizer = RowColSizer()

        self.nr = nr = 1    ## self.network. initialization row start
        self.tr = tr = nr+3 ## training limits and status row start
        self.cr = cr = tr+7 ## canvas row start

        ## network creation
        sizer.Add(wx.StaticText(self, -1, 'Pattern size'), flag=wx.EXPAND, pos=(nr+0,0), border=5)
        self.tc_pat = wx.TextCtrl(self, ID_PatternSize, '100')
        sizer.Add(self.tc_pat, flag=wx.EXPAND, pos=(nr+1,0), border=5)

        sizer.Add(wx.StaticText(self, -1, 'Context size'), flag=wx.EXPAND, pos=(nr+0,1), border=5)
        self.tc_con = wx.TextCtrl(self, ID_ContextSize, '50')
        sizer.Add(self.tc_con, flag=wx.EXPAND, pos=(nr+1,1), border=5)

        sizer.Add(wx.StaticText(self, -1, 'Learning rate'), flag=wx.EXPAND, pos=(nr+0,2), border=5)
        self.tc_lr = wx.TextCtrl(self, ID_LearningRate, '0.01')
        sizer.Add(self.tc_lr, flag=wx.EXPAND, pos=(nr+1,2), border=5)

        sizer.Add(wx.StaticText(self, -1, 'Num Sequences'), flag=wx.EXPAND, pos=(nr+0,3), border=5)
        self.sc_seq = wx.SpinCtrl(self, ID_NumSequences, '2')
        self.sc_seq.SetRange(1, 4)
        sizer.Add(self.sc_seq, flag=wx.EXPAND, pos=(nr+1,3), border=5)

        b_create = wx.Button(self, ID_CreateNetwork, 'Create')
        sizer.Add(b_create, flag=wx.EXPAND, pos=(nr+2,3), border=5)
        self.Bind(wx.EVT_BUTTON, self.OnCreateNetwork, b_create)

        ## start training and stop
        b_train = wx.Button(self, ID_Train, 'Train')
        sizer.Add(b_train, flag=wx.EXPAND, pos=(self.nr+0,4), rowspan=1, border=5)
        self.Bind(wx.EVT_BUTTON, self.OnTrain, b_train)

        b_stop = wx.Button(self, ID_Stop, 'Pause')
        sizer.Add(b_stop, flag=wx.EXPAND, pos=(self.nr+0,5), rowspan=1, border=5)
        wx.EVT_BUTTON(self, ID_Stop, self.OnStop)

        ## autosaving networks
        self.cb_autosave = wx.ComboBox(self, ID_AutoSave,
                                       value="Autosave OFF",
                                       choices=["Autosave ON","Autosave OFF"],
                                       style=wx.CB_DROPDOWN | wx.CB_READONLY)
        sizer.Add(self.cb_autosave, flag=wx.EXPAND, pos=(nr+2,4), border=5)
        self.Bind(wx.EVT_TEXT, self.ComboEvtText, self.cb_autosave)
        
        self.sc_testn = wx.SpinCtrl(self, ID_TestN, str(random.randrange(0, 100)))
        self.sc_testn.SetRange(0, 999)
        self.sc_testn.Show()
        sizer.Add(self.sc_testn, flag=wx.EXPAND, pos=(nr+2,5), border=5)

        self.cb_seqtype = wx.ComboBox(self, ID_SequenceType,
                                      value="Random Sequence", ## "Random Sequence",
                                      choices=["Random Sequence","Controlled Sequence","Ratio Sequence"],
                                      style=wx.CB_DROPDOWN | wx.CB_READONLY)
        self.Bind(wx.EVT_TEXT, self.OnChangeSequenceType, self.cb_seqtype)
        sizer.Add(self.cb_seqtype, flag=wx.EXPAND, pos=(nr+2,0), colspan=3, border=5)

        ## sequence number dependent controls
        ## use sequence name as key
        self.cb_enf = {}
        self.tc_lrf = {}
        self.tc_rlm = {} ## run limit
        self.st_num = {} ## num epochs run
        self.st_sse = {} ## sse
        self.static = {} ## static texts

        self.static['enf'] = wx.StaticText(self, -1, "Self Enforce")
        self.static['lrf'] = wx.StaticText(self, -1, "LR Factor")
        self.static['run'] = wx.StaticText(self, -1, "Epoch Run Limit")
        self.static['epo'] = wx.StaticText(self, -1, 'Trained Epochs')
        self.static['sse'] = wx.StaticText(self, -1, 'Current SSE')
        
        self.static['enf'].Hide()
        self.static['lrf'].Hide()
        self.static['run'].Hide()
        self.static['epo'].Hide()
        self.static['sse'].Hide()

        sizer.Add(self.static['enf'], flag=wx.EXPAND, pos=(self.tr+1,0), border=5)
        sizer.Add(self.static['lrf'], flag=wx.EXPAND, pos=(self.tr+2,0), border=5)
        sizer.Add(self.static['run'], flag=wx.EXPAND, pos=(self.tr+3,0), border=5)
        sizer.Add(self.static['epo'], flag=wx.EXPAND, pos=(self.tr+4,0), border=5)
        sizer.Add(self.static['sse'], flag=wx.EXPAND, pos=(self.tr+5,0), border=5)

        # Set the layout.
        self.SetSizer(sizer)
        self.SetAutoLayout(True)

        for i in range(4):
            self.add_control(i)

        self.init_training_controls()
    # endof __init__
        

    def init_training_controls(self):
        sizer = self.GetSizer()

        ## gauge (progress bar) visual remaining time...
        self.gauge = wx.Gauge(self, ID_Gauge, style=wx.GA_HORIZONTAL|wx.GA_SMOOTH)
        sizer.Add(self.gauge, flag=wx.EXPAND, pos=(self.cr-1,0), colspan=9, border=5)
        self.gauge.Hide()
        
        ## create canvas for showing training info
        self.fig = Figure((1,1), 75) ## (5,10), 50)
        self.canvas = FigureCanvas(self, -1, self.fig)
        self.toolbar = Toolbar(self.canvas)
        self.toolbar.Realize()

        # On Windows, default frame size behaviour is incorrect
        # you don't need this under Linux
        tw, th = self.toolbar.GetSizeTuple()
        fw, fh = self.canvas.GetSizeTuple()
        self.toolbar.SetSize(wxSize(fw, th))

        # Create a figure manager to manage things
        self.figmgr = FigureManager(self.canvas, 1, self)

        # Now put all into the sizer
        sizer.Add(self.canvas, flag=wx.EXPAND, pos=(self.cr,0), size=(10,9), border=5)## colspan=7) ## size=(3,8))
        sizer.Add(self.toolbar, flag=wx.BOTTOM, pos=(self.cr+10,0), colspan=7, border=5)

        sizer.AddGrowableCol(8)
        sizer.AddGrowableRow(self.cr)

    # endof __init__


    def callback(self):
        """ Let the running evolution tell us when a generation is done """
        wx.PostEvent(self, CallBackEvent())

    def OnCallBack(self, event):
        self.update_time_spent()
        self.update_graph()


    def ComboEvtText(self, event):
        if self.cb_autosave.GetValue() == "Autosave ON":
            self.sc_testn.Enable()
        else:
            self.sc_testn.Disable()

    def OnCreateNetwork(self, event):
        self.statusbar.SetStatusText("Creating Network...")

        patternSize = int(self.tc_pat.GetValue())
        contextSize = int(self.tc_con.GetValue())
        learning_rate = float(self.tc_lr.GetValue())
        numSequences = int(self.sc_seq.GetValue())
        self.network = RsrnMs.RSRNMS(patternSize,
                                     contextSize,
                                     learning_rate=learning_rate)

        ## show valid
        for i in range(numSequences):
            ambiguity = [(3,7), (2,5), (3,8), (2,7)]
            seqtypestr = self.cb_seqtype.GetValue()
            if seqtypestr == "Controlled Sequence": 
                self.network.createPatternSequenceDistributed(sequence=global_sequenceNames[i],
                                                              numPatterns=10,
                                                              ambiguity=ambiguity[i])
            elif seqtypestr == "Random Sequence": 
                self.network.createPatternSequenceRandom(sequence=global_sequenceNames[i],
                                                         numPatterns=10,
                                                         ambiguity=ambiguity[i])
            elif seqtypestr == "Ratio Sequence": 
                self.network.createPatternSequenceRatio(sequence=global_sequenceNames[i],
                                                        numPatterns=10,
                                                        ambiguity=ambiguity[i])
            else:
                print 'no sequence type'
            ## make visible
            self.show_control(number=i)
            
        ## hide non-valid
        for i in range(numSequences, 4):
            seq = global_sequenceNames[i]
            self.cb_enf[seq].Hide()
            self.tc_lrf[seq].Hide()
            self.tc_rlm[seq].Hide() 
            self.st_num[seq].Hide()
            self.st_sse[seq].Hide() 
            self.static[seq].Hide() 
            

        self.statusbar.SetStatusText("Network created", 0)
        self.statusbar.SetStatusText('File Created, Not Saved' , 1)
        
        self.init_plot_data()
        if self.cb_autosave.GetValue() == "Autosave ON":
            self.AutoSave(state=-1)
    # endof OnCreateNetwork


    def add_control(self, number=-1):
        if number < 0: return

        sizer = self.GetSizer()
        seq = global_sequenceNames[number]

        ## column heading
        self.static[seq] = wx.StaticText(self, -1, 'Sequence %s' % (seq, ))
        self.static[seq].SetForegroundColour(global_foreground_color[number])
        self.static[seq].Hide() 
        sizer.Add(self.static[seq],
                  flag=wx.EXPAND, pos=(self.tr+0,1+number), border=5)

        self.cb_enf[seq] = wx.ComboBox(self, ID_SelfEnforceList[number],
                                       value=global_self_enforce_method[number],
                                       choices=["Nothing","Self Enforce","PseudoStore 20","PseudoStore 50","PseudoStore 100"],
                                       style=wx.CB_DROPDOWN | wx.CB_READONLY)
        self.Bind(wx.EVT_TEXT, self.OnChangeEnforceMethod, self.cb_enf[seq])

        self.tc_lrf[seq] = wx.TextCtrl(self, ID_LRFactorList[number], '%.2f' % global_lr_factor[number])
        self.tc_rlm[seq] = wx.TextCtrl(self, ID_RunLimitList[number], str(global_runlimit[number]))
        self.st_num[seq] = wx.StaticText(self, ID_NumEpochsList[number], str('  0')) ## , style=wx.TE_READONLY)
        self.st_sse[seq] = wx.StaticText(self, ID_SSEList[number], str('  0')) ## , style=wx.TE_READONLY) 

        self.cb_enf[seq].Hide()
        self.tc_lrf[seq].Hide()
        self.tc_rlm[seq].Hide() 
        self.st_num[seq].Hide()
        self.st_sse[seq].Hide() 

        sizer.Add(self.cb_enf[seq], flag=wx.EXPAND, pos=(self.tr+1,1+number), border=5)
        sizer.Add(self.tc_lrf[seq], flag=wx.EXPAND, pos=(self.tr+2,1+number), border=5)
        sizer.Add(self.tc_rlm[seq], flag=wx.EXPAND, pos=(self.tr+3,1+number), border=5)
        sizer.Add(self.st_num[seq], flag=wx.EXPAND, pos=(self.tr+4,1+number), border=5)
        sizer.Add(self.st_sse[seq], flag=wx.EXPAND, pos=(self.tr+5,1+number), border=5)
    # endof add_control    

    def show_control(self, number=-1):
        ## show labels
        if number == 1:
            self.static['enf'].Show()
            self.static['lrf'].Show()
            self.static['run'].Show()
            self.static['epo'].Show()
            self.static['sse'].Show()

        ## set values
        seq = global_sequenceNames[number]
        self.cb_enf[seq].SetValue(global_se_methods[global_self_enforce[number]])
        self.tc_lrf[seq].SetValue('%.2f' % (global_lr_factor[number], ))
        self.tc_rlm[seq].SetValue(str(global_runlimit[number]))
        self.st_num[seq].SetLabel('  '+str(self.network.numEpochs[seq]))
        self.st_sse[seq].SetLabel('  '+str(self.network.sse(sequence=seq)))

        ## show sequence controls
        self.cb_enf[seq].Show()
        self.tc_lrf[seq].Show()
        self.tc_rlm[seq].Show()
        self.st_num[seq].Show()
        self.st_sse[seq].Show()
        self.static[seq].Show()
    # endof show_control



    # functions used to print debug info
    def OnChangeEnforceMethod(self, evt):
        if 1: return
        for b in self.cb_enf.keys():
            i = 'ABCDEFGH'.index(b)
            global_self_enforce[i] = global_se_methods.index(self.cb_enf[b].GetValue())
            global_self_enforce_method[i] = self.cb_enf[b].GetValue()

    def OnToggle(self, evt):
        if 0: print 'toggle'
             
    def OnChangeSequenceType(self, event):
        if 0: print 'sequence change:', self.cb_seqtype.GetValue() ## GetLabel()
    # endof: functions used to print debug info

        
    def OnStop(self, event):
        self.statusbar.SetStatusText("Pausing training...")
        self.network.run = False
        self.statusbar.SetStatusText("Training paused")


    def OnTrain(self, event, sequence=0):
        if not self.network: return
        if self.network.run: return

        self.network.run = True
        thread.start_new_thread(self.Train, (event, ))
    # endof OnTrain


    def Train(self, event):
        self.statusbar.SetStatusText("Training in progress...")
        self.gauge.Show()
        self.network.run = True
        self.network.learning_rate = float(self.tc_lr.GetValue())
        runlimits = []
        lr_factors = []
        for seq in global_sequenceNames[:self.network.numSequences]:
            runlimits.append(int(self.tc_rlm[seq].GetValue()))
            lr_factors.append(float(self.tc_lrf[seq].GetValue()))
        print 'training'
        print 'runlimits ', runlimits
        print 'enforce   ', global_self_enforce[:self.network.numSequences]
        print 'lr_factors', lr_factors
        
        for number in range(self.network.numSequences):
            seq = global_sequenceNames[number]
            runlimit = global_runlimit[number] = runlimits[number]

            ## check if pseudopatterns should be added to list
            if seq != 'A' and not self.network.numEpochs[seq] and runlimit:
                train_str = self.cb_enf[seq].GetValue()
                if 'PseudoStore' not in train_str:
                    self.network.use_collected_pseudoPatterns = False
                else:
                    numPat = int(train_str.split(' ')[-1]) # fetch the storage size at the end of training method name
                    if True or numPat != len(self.network.pseudoPatternsList):
                        print 'collect pseudopatterns'
                        self.network.collectPseudoPatterns(numPatterns=numPat,
                                                           repetitions=5, fliprate=0)
                    else: print 'collected pseudopatterns exist'
            
            runlimit = global_runlimit[number] = runlimits[number]
            
            lr_factor = global_lr_factor[number] = lr_factors[number]
            current = self.network.numEpochs[seq]

            self.gauge.SetRange(runlimit)
            self.gauge.SetValue(current)

            last_sequence_is_trained_on = False
            while self.network.run and current < runlimit:
                last_sequence_is_trained_on = True ## used to see if training has been STOPPED
                self.network.train(sequence=seq,
                                   numEpochs=10,
                                   self_enforce=global_self_enforce[number],
                                   lr_factor=lr_factor, 
                                   repetitions=5)
                current = self.network.numEpochs[seq]
                self.gauge.SetValue(current)

                ## update screen
                self.update_graph()
                
            if (current >= runlimit and last_sequence_is_trained_on and self.cb_autosave.GetValue() == "Autosave ON"):
                self.AutoSave(state=number)

        self.gauge.Hide()
        self.network.run = False
        self.statusbar.SetStatusText("Training done")
    # endof Train

    def AutoSave(self, state=-2):
        if state < -1: return

        savename = 'GUI_T%s' % (str(self.sc_testn.GetValue()).rjust(3, '0'), )

        for i in range(self.network.numSequences):
            seq = global_sequenceNames[i]
            if self.network.numEpochs[seq]:
                savename += '_%s%s' % (seq,
                                       str(self.network.numEpochs[seq]).rjust(3, '0'))

        savename += '.rsrnms'
        self.network.saveNetwork(savefilename=savename,
                                 complete=False)
    # endof AutoSave


    def GetToolBar(self):
        # You will need to override GetToolBar if you are using an 
        # unmanaged toolbar in your frame
        return self.toolbar
    # endof GetToolBar
    

    ## drawing the graphs
    def init_plot_data(self):
        """ show statistics of the evolution """
        self.cls_max  = 100 ## self.evolution.get_problem_size()

        self.fig.clear()

        ## create subplots for correct units (average)
        if self.network:
            numseq = self.network.numSequences

            self.numcor1_plot = {}
            for i in range(numseq):
                seq = global_sequenceNames[i]
                ## add subplot
                numcor1 = self.fig.add_subplot((self.useoneplot and 111 or 211))
                self.numcor1_plot[seq], = numcor1.plot([0], [0], global_plot_color[i], lw=1.3)

            numcor1.set_xlim([0,205])
            numcor1.set_ylim([-2,int(self.network.patternSize*1.1)])
            numcor1.grid('on')
            numcor1.set_title('Average Correct Units - Limit .1 (#%d)' % (self.network.patternSize, ), fontsize=12)
            numcor1.legend( ('A','B','C','D')[:numseq], 'lower right', pad=.5, handlelen=.02, handletextsep=.01)

            self.numcor1 = numcor1

            ## manage the space surrounding the subplots
            self.fig.subplots_adjust(left=.1, right=.95,
                                     bottom=.1, top=.9,
                                     wspace=.2, hspace=.4)

        # Draw the graph.
        self.canvas.draw()
        self.update_graph()
    # endof init_plot_data


    def update_graph(self):
        if not self.network: return

        # update viewport
        extra = int((self.network.numEpochsTotal-151) / 50)
        if extra < 0: extra = 0

        ## set x limits
        self.numcor1.set_xlim([0, 205+extra*50])

        for i in range(self.network.numSequences):
            seq = global_sequenceNames[i]

            ## get graph data
            cor_ydata1 = self.network.stats['numcor'][seq+'1']
            cor_xdata = range(len(cor_ydata1))

            ## trim the data
            x_step = int((205+extra*50)/200)
            cor_xdata = [cor_xdata[i] for i in range(0, len(cor_xdata), x_step)]
            cor_ydata1 = [cor_ydata1[i] for i in range(0, len(cor_ydata1), x_step)]

            ## update graph data        
            self.numcor1_plot[seq].set_data(cor_xdata, cor_ydata1)

            ## update text controls
            self.st_num[seq].SetLabel('  %d' % (self.network.numEpochs[seq], )) ## SetValue('%d' % (self.network.numEpochs[seq], ))
            self.st_sse[seq].SetLabel('  %.3f' % (self.network.sse(sequence=seq), ))

        # Redraw the graph.
        self.canvas.draw()
        self.canvas.gui_repaint()

        return
    # endof update_graph
            
    
    def OnMenuSave(self, event):
        ## file filters
        wildcard = "Saved RSRN Network (*.rsrnms)|*.rsrnms|"    \
                   "Python source (*.py)|*.py|"     \
                   "All files (*.*)|*.*"

        ## open file dialog
        dlg = wx.FileDialog(self,
                            message="Save file as ...",
                            defaultDir='C:\Python24\MyScripts\savefiles\\',
                            defaultFile="",
                            wildcard=wildcard,
                            style=wx.SAVE)

        ## set default filter
        dlg.SetFilterIndex(0)

        # show the dialog and retrieve the user response. if it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            ## save this network
            self.network.saveNetwork(savefilename=dlg.GetPath(), complete=True)
            self.statusbar.SetStatusText('File saved: ' + dlg.GetPath().split('\\')[-1], 1)
            self.statusbar.SetStatusText('Saving completed', 0)

        # destroy the dialog. 
        dlg.Destroy()

    def OnMenuLoad(self, event):
        ## file filters
        wildcard = "Saved RSRN Network (*.rsrnms)|*.rsrnms|"    \
                   "Python source (*.py)|*.py|"     \
                   "All files (*.*)|*.*"

        ## open file dialog
        dlg = wx.FileDialog(self,
                            message="Choose a file",
                            defaultDir='C:\Python24\MyScripts\savefiles\\',
                            defaultFile="",
                            wildcard=wildcard,
                            style=wx.OPEN | wx.CHANGE_DIR)

        # show the dialog and retrieve the user response. if it is the OK response, 
        # process the data.
        if dlg.ShowModal() == wx.ID_OK:
            ## load the old network
            self.network = RsrnMs.loadNetwork(savefilename=dlg.GetPath(),
                                              complete=True)
            self.statusbar.SetStatusText('File loaded: ' + dlg.GetPath().split('\\')[-1], 1)
            self.statusbar.SetStatusText('Loading completed', 0)

        ## destroy the dialog
        dlg.Destroy()

        self.tc_pat.SetValue(str(self.network.patternSize))
        self.tc_con.SetValue(str(self.network.contextSize))
        self.sc_seq.SetValue(int(self.network.numSequences))
        self.tc_lr.SetValue('%.2f'%self.network.learning_rate)

        self.init_plot_data()

        ## show valid
        for i in range(self.network.numSequences):
            ## make visible
            self.show_control(number=i)
            seq = 'ABCDEFG'[i]
            try:
                self.cb_seqtype.SetValue(self.network.sequenceTypes[seq])
                train_tuple = self.network.sequenceTrain[seq]
                self.cb_enf[seq].SetValue(train_tuple[0])
                self.tc_lrf[seq].SetValue(str(train_tuple[1]))
            except Exception, e:
                print 'Load Error: Old Network Type'
            
        ## hide non-valid
        for i in range(self.network.numSequences, 4):
            seq = global_sequenceNames[i]
            self.cb_enf[seq].Hide()
            self.tc_lrf[seq].Hide()
            self.tc_rlm[seq].Hide() 
            self.st_num[seq].Hide()
            self.st_sse[seq].Hide() 
            self.static[seq].Hide() 

    # endof OnMenuLoad
        
# endof RsrnMsPanel
        

class RsrnMsFrame(wx.Frame):
    def __init__(self, parent, ID, title):
        wx.Frame.__init__(self, parent, ID, title, pos=(50,50), size=(0,0),
                          style=wx.NO_FULL_REPAINT_ON_RESIZE|wx.DEFAULT_FRAME_STYLE)
        try:self.SetIcon(wx.Icon("py.ico", wx.BITMAP_TYPE_ICO))
        except: pass

        self.CreateStatusBar()
        self.SetStatusText("Ready to Create or Load networks") 

        ## file menu
        filemenu = wx.Menu()
        filemenu.Append(ID_MenuNew, "&New\tCtrl-N", "Create a new network")
        filemenu.AppendSeparator()
        filemenu.Append(ID_MenuSave, "&Save\tCtrl-S", "Save the existing network to file")
        filemenu.Append(ID_MenuLoad, "&Load\tCtrl-L", "Load a network from file")
        filemenu.AppendSeparator()
        filemenu.Append(ID_MenuExit, "E&xit\tAlt-F4", "Exit the program without saving")

        ## add event handlers
        wx.EVT_MENU(self, ID_MenuNew, self.OnMenuNew)
        wx.EVT_MENU(self, ID_MenuSave, self.OnMenuSave)
        wx.EVT_MENU(self, ID_MenuLoad, self.OnMenuLoad)
        wx.EVT_MENU(self, ID_MenuExit, self.OnMenuExit)

        ## train menu
        trainmenu = wx.Menu()
        trainmenu.Append(ID_Train, "Train Net &A\tAlt-T", "Train Net")
        trainmenu.AppendSeparator()
        trainmenu.Append(ID_Stop, "&Pause training\tAlt-P", "Pause training")

        ## event handlers should exist?
        wx.EVT_MENU(self, ID_Train, self.OnTrain)
        wx.EVT_MENU(self, ID_Stop, self.OnStop)
        
        ## connect menu items
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu, "&File")
        menuBar.Append(trainmenu, "&Training")
        self.SetMenuBar(menuBar)
        self.Show(True)
        wx.EVT_CLOSE(self, self.OnCloseFrame)

        ## create the panel
        win = RsrnMsPanel(self, Log())

        sizer = win.GetSizer()
        b_close = wx.Button(win, ID_Close, "Close")
        sizer.Add(b_close, flag=wx.EXPAND, pos=(win.nr+1,5), colspan=1, border=5)
        self.Bind(wx.EVT_BUTTON, self.OnCloseMe, b_close)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        win.statusbar = self.GetStatusBar()
        win.statusbar.SetFieldsCount(2)


        ## handle closing
        wx.EVT_CLOSE(win, self.OnCloseFrame)
        self.SetSize((win.useoneplot and (800,640) or (800,700)))
        self.SetPosition((10,30))
        win.SetFocus()
        self.window = win
        self.window.init_plot_data()


    ## functions for closing the frame
    def OnCloseMe(self, event):
        self.Close(True)

    def OnCloseWindow(self, event):
        self.Destroy()
        
                       
    def OnMenuExit(self, event):
        self.Close(True)

    def OnCloseFrame(self, event):
        if hasattr(self, "window") and hasattr(self.window, "ShutdownDemo"):
            self.window.ShutdownDemo()
        event.Skip()

    def OnCloseWindow(self, event):
        self.Destroy()

    ## functions for new, save and open
    def OnMenuNew(self, event):
        self.window.OnCreateNetwork(event)
        
    def OnMenuSave(self, event):
        self.window.OnMenuSave(event)

    def OnMenuLoad(self, event):
        self.window.OnMenuLoad(event)

    ## functions for transferring training calls
    def OnTrain(self, event):
        self.window.OnTrain(event)

    def OnStop(self, event):
        self.window.OnStop(event)


class MyApp(wx.App):
    def __init__(self, name):
        self.name = name
        wx.App.__init__(self, 0)
        

    def OnInit(self):
        wx.InitAllImageHandlers()
        wx.Log_SetActiveTarget(wx.LogStderr())

        frame = RsrnMsFrame(None, -1, self.name)

        self.SetTopWindow(frame)
        self.frame = frame
        return True

def main():
    app = MyApp("RSRN with Multiple Sequences") ## and Self Enforcing")
    app.MainLoop()

if __name__ == "__main__":
    main()
